const Promise = require('bluebird');
const read = require('../../util/forum/read');
const { matrixPools } = require('../../util/db');

const computeAuthorWeights = async (batchItems_) => {
  const weights = {};
  await Promise.each(batchItems_, async (postId) => {
    const post = await read(postId);
    const matrixPool = await matrixPools.get(postId);
    const { fee, result: { votePasses, quorumMet } } = matrixPool;
    post.authors.forEach(({ authorAddress, weightPPM }) => {
      weights[authorAddress] = weights[authorAddress] ?? 0;
      if (votePasses && quorumMet) {
        // scale by matrix pool outcome and strength
        weights[authorAddress] += weightPPM * fee;
      }
      // TODO: Rewards for policing
      // TODO: Propagation via references
    });
  });
  // Rescale author weights so they sum to 1000000
  const sumOfWeights = Object.values(weights).reduce((t, v) => t + v, 0);
  const scaledWeights = Object.values(weights)
    .map((weight) => Math.floor((weight * 1000000) / sumOfWeights));
  const sumOfScaledWeights = Object.values(scaledWeights).reduce((t, v) => t + v, 0);
  scaledWeights[0] += 1000000 - sumOfScaledWeights;
  const authors = Object.keys(weights)
    .map((authorAddress, i) => ({ authorAddress, weightPPM: scaledWeights[i] }));
  return authors;
};

module.exports = computeAuthorWeights;
